from yafowil.base import factory
from yafowil.utils import entry_point
import os


resourcedir = os.path.join(os.path.dirname(__file__), 'resources')
js = [{
    'group': 'yafowil.widget.slider.dependencies',
    'resource': 'jquery.ui.slider.min.js',
    'order': 20,
}, {
    'group': 'yafowil.widget.slider.common',
    'resource': 'widget.js',
    'order': 21,
}]
default_css = [{
    'group': 'yafowil.widget.slider.dependencies',
    'resource': 'jquery.ui.slider.css',
    'order': 20,
}, {
    'group': 'yafowil.widget.slider.common',
    'resource': 'widget.css',
    'order': 21,
}]
bootstrap_css = [{
    'group': 'yafowil.widget.slider.dependencies',
    'resource': 'jquery.ui.slider.bootstrap.css',
    'order': 20,
}, {
    'group': 'yafowil.widget.slider.common',
    'resource': 'widget.css',
    'order': 21,
}]
plone5_css = [{
    'group': 'yafowil.widget.slider.dependencies',
    'resource': 'jquery.ui.slider.plone5.css',
    'order': 20,
}, {
    'group': 'yafowil.widget.slider.common',
    'resource': 'widget.css',
    'order': 21,
}]


@entry_point(order=10)
def register():
    import widget
    factory.register_theme('default', 'yafowil.widget.slider',
                           resourcedir, js=js, css=default_css)
    factory.register_theme('bootstrap', 'yafowil.widget.slider',
                           resourcedir, js=js, css=bootstrap_css)
    factory.register_theme('plone5', 'yafowil.widget.slider',
                           resourcedir, js=js, css=plone5_css)



from node.utils import UNSET
from yafowil.base import factory
from yafowil.base import fetch_value
from yafowil.utils import attr_value
from yafowil.utils import cssclasses
from yafowil.utils import cssid
from yafowil.utils import data_attrs_helper
from yafowil.utils import managedprops


@managedprops('range')
def slider_extractor(widget, data):
    # extract range tuple
    if attr_value('range', widget, data) is True:
        lower_value_name = '%s.lower' % widget.dottedpath
        upper_value_name = '%s.upper' % widget.dottedpath
        lower_value = UNSET
        if lower_value_name in data.request:
            lower_value = data.request[lower_value_name]
            if lower_value:
                lower_value = int(lower_value)
            else:
                lower_value = UNSET
        upper_value = UNSET
        if upper_value_name in data.request:
            upper_value = data.request[upper_value_name]
            if upper_value:
                upper_value = int(upper_value)
            else:
                upper_value = UNSET
        return [lower_value, upper_value]
    # regular value extraction
    if widget.dottedpath in data.request:
        val = data.request[widget.dottedpath]
        if val:
            return int(val)
    return UNSET


js_options = ['orientation', 'range', 'min', 'max', 'step', 'slide', 'change']

@managedprops(*['show_value', 'unit', 'height', 'data'] + js_options)
def slider_edit_renderer(widget, data):
    value = fetch_value(widget, data)
    content = ''
    range = attr_value('range', widget, data)
    if range is True:
        lower_input_attrs = {
            'type': 'text',
            'name': '%s.lower' % widget.dottedpath,
            'id': cssid(widget, 'input-lower'),
            'style': 'display:none;',
            'class': 'lower_value',
            'value': value and value[0],
        }
        content += data.tag('input', **lower_input_attrs)
        upper_input_attrs = {
            'type': 'text',
            'name': '%s.upper' % widget.dottedpath,
            'id': cssid(widget, 'input-upper'),
            'style': 'display:none;',
            'class': 'upper_value',
            'value': value and value[1],
        }
        content += data.tag('input', **upper_input_attrs)
    else:
        input_attrs = {
            'type': 'text',
            'name': widget.dottedpath,
            'id': cssid(widget, 'input'),
            'style': 'display:none;',
            'class': 'slider_value',
            'value': value,
        }
        content += data.tag('input', **input_attrs)
    show_value = attr_value('show_value', widget, data)
    if show_value:
        unit = attr_value('unit', widget, data)
        if unit:
            content += data.tag('span', '%s: ' % unit, **{'class': 'unit'})
        if range is True:
            content += data.tag('span', value[0], **{'class': 'lower_value'})
            content += ' - '
            content += data.tag('span', value[1], **{'class': 'upper_value'})
        else:
            content += data.tag('span', value, **{'class': 'slider_value'})
    slider_attrs = {'class': 'slider'}
    if attr_value('orientation', widget, data) == 'vertical':
        height = attr_value('height', widget, data)
        if height:
            slider_attrs['style'] = 'height:%spx;' % height
    content += data.tag('div', ' ', **slider_attrs)
    wrapper_attrs = data_attrs_helper(widget, data, js_options)
    wrapper_attrs['class'] = cssclasses(widget, data)
    html_data = widget.attrs['data']
    data_keys = html_data.keys()
    for key in data_keys:
        if key in js_options:
            raise ValueError(u"Additional data dict contains reserved "
                             u"attribute name '%s'" % key)
        wrapper_attrs['data-%s' % key] = html_data[key]
    return data.tag('div', content, **wrapper_attrs)


def slider_display_renderer(widget, data):
    raise NotImplementedError(u"``yafowil.widget.slider`` does not support "
                              u"display mode yet")


factory.register(
    'slider',
    extractors=[slider_extractor],
    edit_renderers=[slider_edit_renderer],
    display_renderers=[slider_display_renderer])

factory.doc['blueprint']['slider'] = \
"""Add-on blueprint `yafowil.widget.slider
<http://github.com/bluedynamics/yafowil.widget.slider/>`_ .
"""

factory.defaults['slider.default'] = ''

factory.defaults['slider.class'] = 'yafowil_slider'

factory.defaults['slider.show_value'] = False
factory.doc['props']['slider.show_value'] = \
"""Show value in addition to slider.
"""

factory.defaults['slider.unit'] = ''
factory.doc['props']['slider.unit'] = \
"""Slider value unit.
"""

factory.defaults['slider.orientation'] = None
factory.doc['props']['slider.orientation'] = \
"""Slider Orientation. Either ``horizontal`` or ``vertical``.
"""

factory.defaults['slider.height'] = None
factory.doc['props']['slider.height'] = \
"""Height of slider if orientation is ``vertical`` in pixel.
"""

factory.defaults['slider.range'] = None
factory.doc['props']['slider.range'] = \
"""Slider Range. Either ``True``, ``'min'`` or ``'max'``.
"""

factory.defaults['slider.min'] = None
factory.doc['props']['slider.min'] = \
"""Minimum slider value. Defaults to 0.
"""

factory.defaults['slider.max'] = None
factory.doc['props']['slider.max'] = \
"""Maximum slider value value. Defaults to 100.
"""

factory.defaults['slider.step'] = None
factory.doc['props']['slider.step'] = \
"""Snap slider to increments.
"""

factory.defaults['slider.slide'] = None
factory.doc['props']['slider.slide'] = \
"""Optional Javascript ``slide`` callback as string.
"""

factory.defaults['slider.change'] = None
factory.doc['props']['slider.change'] = \
"""Optional Javascript ``change`` callback as string.
"""

factory.defaults['slider.data'] = dict()
factory.doc['props']['slider.data'] = \
"""Additional data redered as HTML data attributes on slider wrapper
DOM Element.
"""



from yafowil.base import factory


DOC_DEFAULT_SLIDER = """
Default slider
--------------

jQuery UI slider. Default behavior.

.. code-block:: python

    slider = factory('#field:slider', value=20, props={
        'label': 'Default Slider',
    })
"""

def default_slider():
    form = factory('fieldset',
                   name='yafowil.widget.slider.default')
    slider = form['slider'] = factory('#field:slider', value=20, props={
        'label': 'Default Slider',
    })
    return {
        'widget': form,
        'doc': DOC_DEFAULT_SLIDER,
        'title': 'Default slider',
    }


DOC_FIXED_MIN_RANGE_SLIDER = """
Range with fixed minimum
------------------------

Fix the minimum value of the range slider so that the user can only select a
maximum. Set the ``range`` option to ``'min'``.

.. code-block:: python

    slider = factory('#field:slider', value=37, props={
        'label': 'Range with fixed minimum',
        'range': 'min',
        'min': 1,
        'max': 200,
        'show_value': True,
        'unit': 'Pieces',
    })
"""

def fixed_minimum_range():
    form = factory('fieldset',
                   name='yafowil.widget.slider.fixed_minimum_range')
    slider = form['slider'] = factory('#field:slider', value=37, props={
        'label': 'Range with fixed minimum',
        'range': 'min',
        'min': 1,
        'max': 200,
        'show_value': True,
        'unit': 'Pieces',
    })
    return {
        'widget': form,
        'doc': DOC_FIXED_MIN_RANGE_SLIDER,
        'title': 'Fixed minimum',
    }


DOC_FIXED_MAX_RANGE_SLIDER = """
Range with fixed maximum
------------------------

Fix the maximum value of the range slider so that the user can only select a
minimum. Set the ``range`` option to ``'max'``.

.. code-block:: python

    slider = factory('#field:slider', value=2, props={
        'label': 'Range with fixed maximum',
        'range': 'max',
        'min': 1,
        'max': 10,
        'show_value': True,
        'unit': 'Minimum number',
    })
"""

def fixed_maximum_range():
    form = factory('fieldset',
                   name='yafowil.widget.slider.fixed_maximum_range')
    slider = form['slider'] = factory('#field:slider', value=2, props={
        'label': 'Range with fixed maximum',
        'range': 'max',
        'min': 1,
        'max': 10,
        'show_value': True,
        'unit': 'Minimum number',
    })
    return {
        'widget': form,
        'doc': DOC_FIXED_MAX_RANGE_SLIDER,
        'title': 'Fixed maximum',
    }


DOC_RANGE_SLIDER = """
Range slider
------------

Set the range option to true to capture a range of values with two drag
handles. The space between the handles is filled with a different background
color to indicate those values are selected.

.. code-block:: python

    slider = factory('#field:slider', value=[75, 300], props={
        'label': 'Range slider',
        'range': True,
        'min': 0,
        'max': 500,
        'show_value': True,
        'unit': 'Price range (EUR)',
    })
"""

def range_slider():
    form = factory('fieldset',
                   name='yafowil.widget.slider.range')
    slider = form['slider'] = factory('#field:slider', value=[75, 300], props={
        'label': 'Range slider',
        'range': True,
        'min': 0,
        'max': 500,
        'show_value': True,
        'unit': 'Price range (EUR)',
    })
    return {
        'widget': form,
        'doc': DOC_RANGE_SLIDER,
        'title': 'Range slider',
    }


DOC_STEP_SLIDER = """
Step slider
-----------

Increment slider values with the step option set to an integer, commonly a
dividend of the slider's maximum value. The default increment is '1'.

.. code-block:: python

    slider = factory('#field:slider', value=100, props={
        'label': 'Step slider',
        'min': 0,
        'max': 500,
        'step': 50,
        'show_value': True,
        'unit': 'Donation (50 EUR increments)',
    })
"""

def step_slider():
    form = factory('fieldset',
                   name='yafowil.widget.slider.step')
    slider = form['slider'] = factory('#field:slider', value=100, props={
        'label': 'Step slider',
        'min': 0,
        'max': 500,
        'step': 50,
        'show_value': True,
        'unit': 'Donation (50 EUR increments)',
    })
    return {
        'widget': form,
        'doc': DOC_STEP_SLIDER,
        'title': 'Step slider',
    }


DOC_VERTICAL_SLIDER = """
Vertical slider
---------------

Change the orientation of the slider to vertical. Assign a height value via
``height`` or by setting the height through CSS, and set the ``orientation``
option to ``vertical``.

.. code-block:: python

    slider = factory('#field:slider', value=50, props={
        'label': 'Vertical slider',
        'orientation': 'vertical',
        'show_value': True,
        'height': 200,
        'unit': 'mmHg',
    })
"""

def vertical_slider():
    form = factory('fieldset',
                   name='yafowil.widget.slider.vertical')
    slider = form['slider'] = factory('#field:slider', value=50, props={
        'label': 'Vertical slider',
        'orientation': 'vertical',
        'show_value': True,
        'height': 200,
        'unit': 'mmHg',
    })
    return {
        'widget': form,
        'doc': DOC_VERTICAL_SLIDER,
        'title': 'Vertical slider',
    }


def get_example():
    return [
        default_slider(),
        fixed_minimum_range(),
        fixed_maximum_range(),
        range_slider(),
        step_slider(),
        vertical_slider(),
    ]



from setuptools import find_packages
from setuptools import setup
import os


version = '1.2'
shortdesc = 'Slider Widget for YAFOWIL'
longdesc = open(os.path.join(os.path.dirname(__file__), 'README.rst')).read()
longdesc += open(os.path.join(os.path.dirname(__file__), 'HISTORY.rst')).read()
longdesc += open(os.path.join(os.path.dirname(__file__), 'LICENSE.rst')).read()
tests_require = ['yafowil[test]']


setup(
    name='yafowil.widget.slider',
    version=version,
    description=shortdesc,
    long_description=longdesc,
    classifiers=[
        'Environment :: Web Environment',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'License :: OSI Approved :: BSD License',
    ],
    keywords='',
    author='BlueDynamics Alliance',
    author_email='dev@bluedynamics.com',
    url=u'http://pypi.python.org/pypi/yafowil.widget.slider',
    license='Simplified BSD',
    packages=find_packages('src'),
    package_dir = {'': 'src'},
    namespace_packages=['yafowil', 'yafowil.widget'],
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        'setuptools',
        'yafowil>2.1.99',
    ],
    tests_require=tests_require,
    extras_require = dict(
        test=tests_require,
    ),
    test_suite="yafowil.widget.slider.tests.test_suite",
    entry_points={
        'yafowil.plugin': [
            'register = yafowil.widget.slider:register',
            'example = yafowil.widget.slider.example:get_example'
        ],
    })



from interlude import interact
from pprint import pprint
from yafowil.tests import pxml
import doctest
import unittest


optionflags = doctest.NORMALIZE_WHITESPACE | \
              doctest.ELLIPSIS | \
              doctest.REPORT_ONLY_FIRST_FAILURE

TESTFILES = [
    'widget.rst',
]


def test_suite():
    return unittest.TestSuite([
        doctest.DocFileSuite(
            file, 
            optionflags=optionflags,
            globs={'interact': interact,
                   'pprint': pprint,
                   'pxml': pxml},
        ) for file in TESTFILES
    ])


if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')                 #pragma NO COVER
